// this class prints gst registration 

const escpos = require('escpos');
const moment = require('moment');
const currencyFormatter = require('currency-formatter');

function print(dbPrinter, dbPrintout, _receipt, callback) {
    var printout = {};// require('../terminalInfo.json');
    var hasPrinter = false;
    var receipt = _receipt;
    var paperMaxWidth = dbPrinter.paperWidth;
    var printout = dbPrintout;
    var device;

    //setup
    if (dbPrinter.connectionType === 'USB') {
        try {
            device = new escpos.USB();
            hasPrinter = true;
        } catch (err) {
            console.log('No USB Printer found');
        }
    } else if (dbPrinter.connectionType === 'SERIAL') {
        device = new escpos.Serial(dbPrinter.directory, {
            baudRate: 38400,
            autoOpen: false
        });
        hasPrinter = true;
    } else if (dbPrinter.connectionType === 'NET') {
        device = new escpos.Network(dbPrinter.directory);
        hasPrinter = true;
    } else {
        var err = new Error('##printer is not defined in printerDB');
        return callback(err);
    }

    if (hasPrinter) {
        printer = new escpos.Printer(device);

        device.open(function (error) {
            if (error) {
                console.log('cant print');
                //cant return error as the async might stop
                return callback();
            }
            // terminalInfo = require('../terminalInfo.json');
            printHeader(printer, paperMaxWidth, printout);
            printBody(printer, paperMaxWidth, receipt);
            printPaymentInfo(printer, paperMaxWidth, receipt);
            printFooter(printer, paperMaxWidth, printer, receipt);
            close(printer);
            callback();
        });
    }

}
function printHeader(printer, paperMaxWidth, printout) {

    printer
        .font('a')
        .align('ct')
        .style('normal');

    if (hasStringDataInJson(printout.companyName)) {
        printer.text(printout.companyName);
    }
    if (hasStringDataInJson(printout.addressOne)) {
        printer.text(printout.addressOne);
    }
    if (hasStringDataInJson(printout.addressTwo)) {
        printer.text(printout.addressTwo);
    }
    if (hasStringDataInJson(printout.postalCode)) {
        printer.text(printout.postalCode);
    }
    if (hasStringDataInJson(printout.gstRegNo)) {
        printer.text(printout.gstRegNo);
    }
    printer.text('');
}

function printBody(printer, paperMaxWidth, receipt) {

    printer
        .font('a')
        .align('lt')
        .style('normal')
        .text('Tax Invoice No: ' + receipt.payment.idReceipts)
        .text(dateTimeParser(receipt.issueTime))
        .text('')
    printTable('Qty', 'Description', 'Amount', 'bu', printer, paperMaxWidth);
    printProducts(printer, paperMaxWidth, receipt);

}

function printProducts(printer, paperMaxWidth, receipt) {
    printer
        .font('a')
        .align('lt')
    for (var i = 0; i < receipt.order.orderedProducts.length; i++) {
        printTable(receipt.order.orderedProducts[i].quantity.toString(),
            receipt.order.orderedProducts[i].name,
            currencyFormatter.format(receipt.order.orderedProducts[i].price, { code: 'SGD' }), 'normal', printer, paperMaxWidth);
        if (receipt.order.orderedProducts[i].remark && receipt.order.orderedProducts[i].remark !== '') {
            printSubProducts(receipt.order.orderedProducts[i].remark.split(';'), printer, paperMaxWidth);
        }
    }

}

function printSubProducts(subProducts, printer, paperMaxWidth) {
    for (var i = 0; i < subProducts.length; i++) {
        printTable('', '-- ' + subProducts[i], '', 'normal', printer, paperMaxWidth);
    }
}

function printFooter(printer, paperMaxWidth, printout, receipt) {
    if (receipt.order.idOrders) {
        var queueNo = Math.abs(receipt.order.idOrders) % 1000;
        if (queueNo < 10) {
            queueNo = '00' + queueNo;
        } else if (queueNo < 100) {
            queueNo = '0' + queueNo;
        }
    }

    printer
        .font('a')
        .align('ct')
        .style('normal')
        .size(2, 2)
        .text('')
        .text('QUEUE NO: K' + queueNo)
        .text('')
        .size(1, 1)
        .style('b')
        .text('Please Look Out At The TV Screen'.toUpperCase())
        .text('');
    if (hasStringDataInJson(printout.footerOne)) {
        printer.text(printout.footerOne);
    }
    if (hasStringDataInJson(printout.footerTwo)) {
        printer.text(printout.footerTwo);
    }
    if (hasStringDataInJson(printout.footerThree)) {
        printer.text(printout.footerThree);
    }
    if (hasStringDataInJson(printout.footerFour)) {
        printer.text(printout.footerFour);
    }
    if (hasStringDataInJson(printout.footerFive)) {
        printer.text(printout.footerFive);
    }
    printer.text('');
}
function printPaymentInfo(printer, paperMaxWidth, receipt) {

    const totalText = 'Total';
    const totalAmt = currencyFormatter.format(receipt.payment.paymentAmount, { code: 'SGD' });
    const payableText = 'Payable';
    const receivedText = 'Received';
    const inclusive = 'Inclusive of 7% GST'
    const receivedAmt = currencyFormatter.format(receipt.payment.receivedAmount, { code: 'SGD' });

    printer.text('');
    var ret = receipt.payment.paymentModeName;//.replace(config.ccPaymentHeader, '');
    printer.style('u').text(spaceBuilder(paperMaxWidth));
    printer
        .style('b')
        .text(totalText + spaceBuilder((paperMaxWidth - totalText.length - totalAmt.toString().length)) + totalAmt)
        .style('normal')
        .text(payableText + spaceBuilder((paperMaxWidth - payableText.length - totalAmt.toString().length)) + totalAmt)
        .text(ret + spaceBuilder((paperMaxWidth - ret.length - receivedAmt.toString().length)) + receivedAmt)
        .text(inclusive + spaceBuilder((paperMaxWidth - inclusive.length - (receipt.var9.toString().length + 1))) + '$' + receipt.var9.toString());
    if (receipt.ccinfo) {
        const approvalText = 'Approval Code: ';
        const cardNumText = 'Card Number: ';
        const cardHolderText = 'Card Holder: ';

        printer.style('u').text(spaceBuilder(paperMaxWidth)).style('normal');
        printer
            .text(receipt.ccinfo.cardType)
            .text(approvalText + receipt.ccinfo.approvalCode)
            .text(cardNumText + receipt.ccinfo.cardNum)
            .text(cardHolderText + receipt.ccinfo.cardHolder)
    }

}

function printTable(textOne, textTwo, textThree, style, printer, paperMaxWidth) {
    const colOneMax = 5;
    const colThreeMax = 9;
    var colTwoMax = paperMaxWidth - colOneMax - colThreeMax;
    var isHan = false;
    var strings;

    if (hasChineseCharacter(textTwo)) {
        var count = countChineseChar(textTwo);
        isHan = true;
        strings = splitter(textTwo, (colTwoMax - 11));
    } else {
        strings = splitter(textTwo, colTwoMax);
    }

    printer
        .style(style);

    for (var i = 0; i < strings.length; i++) {
        if (i === 0) {
            if (isHan) {
                printer.text(textOne + spaceBuilder(colOneMax, textOne.length) +
                    strings[i] + spaceBuilder((colTwoMax - countChineseChar(strings[i])), strings[i].length) +
                    spaceBuilder(colThreeMax, textThree.length) + textThree
                );
            } else {
                printer.text(textOne + spaceBuilder(colOneMax, textOne.length) +
                    strings[i] + spaceBuilder(colTwoMax, strings[i].length) +
                    spaceBuilder(colThreeMax, textThree.length) + textThree
                );
            }
        } else {
            printer.text('' + spaceBuilder(colOneMax, ''.length) +
                strings[i] + spaceBuilder(colTwoMax, strings[i].length) +
                spaceBuilder(colThreeMax, ''.length) + ''
            );
        }
    }

}

function spaceBuilder(maxLen, txtLen) {
    var spacing = '';
    txtLen = txtLen || 0;
    var amtSpace = maxLen - txtLen;
    for (var i = 0; i < amtSpace; i++) {
        spacing += ' ';
    }
    return spacing;
}

function splitter(str, l) {
    var strs = [];
    while (str.length > l) {
        var pos = str.substring(0, l).lastIndexOf(' ');
        pos = pos <= 0 ? l : pos;
        strs.push(str.substring(0, pos));
        var i = str.indexOf(' ', pos) + 1;
        if (i < pos || i > pos + l)
            i = pos;
        str = str.substring(i);
    }
    strs.push(str);
    return strs;
}

function close(printer) {
    printer
        .cut(true)
        .close();
}

function dateTimeParser(value) {
    var datetime = moment(new Date(value).getTime());
    return datetime.format('DD/MM/YYYY HH:mm');
}

function hasChineseCharacter(str) {
    var re1 = new RegExp("[\u4E00-\uFA29]+"); //Chinese character range
    str = str.replace(/\s/g, '');
    return re1.test(str);
}

function countChineseChar(str) {
    var count = 0;
    for (var i = 0; i < str.length; i++) {
        if (str.charCodeAt(i) >= 10000) {
            count++;
        }
    }
    return count;
}

function hasStringDataInJson(key) {
    return key && key !== '';
}

module.exports = print;